@@ -14,6 +14,6 @@
 
 """Core functionality for the AWS API MCP server."""
 
-from . import aws, common, data, kb, metadata, parser
+from . import aws, common, data, metadata, parser
 
-__all__ = ['aws', 'common', 'data', 'kb', 'metadata', 'parser']
+__all__ = ['aws', 'common', 'data', 'metadata', 'parser']
