@@ -79,8 +79,8 @@ def translate_cli_to_ir(cli_command: str) -> IRTranslation:
 
 def interpret_command(
     cli_command: str,
-    default_region: str,
     max_results: int | None = None,
+    credentials: Credentials | None = None,
 ) -> InterpretedProgram:
     """Interpret the CLI command.
 
@@ -95,14 +95,14 @@ def interpret_command(
     if translation.command is None:
         return InterpretedProgram(translation=translation)
 
-    region = translation.command.region or default_region
+    region = translation.command.region
     if (
         translation.command.command_metadata.service_sdk_name in GLOBAL_SERVICE_REGIONS
         and region != GLOBAL_SERVICE_REGIONS[translation.command.command_metadata.service_sdk_name]
     ):
         region = GLOBAL_SERVICE_REGIONS[translation.command.command_metadata.service_sdk_name]
 
-    credentials = get_local_credentials(
+    credentials = credentials or get_local_credentials(
         profile=translation.command.profile or AWS_API_MCP_PROFILE_NAME
     )
 
@@ -115,6 +115,7 @@ def interpret_command(
             region=region,
             client_side_filter=translation.command.client_side_filter,
             max_results=max_results,
+            endpoint_url=translation.command.endpoint_url,
         )
     except botocore.exceptions.ClientError as error:
         service_error = str(error)
