@@ -13,13 +13,14 @@
 # limitations under the License.
 
 import contextlib
-from ..aws.services import driver
+from ..aws.services import get_awscli_driver
 from ..common.config import AWS_API_MCP_PROFILE_NAME, DEFAULT_REGION
 from ..common.errors import AwsApiMcpError, Failure
 from ..common.models import (
     AwsApiMcpServerErrorResponse,
     AwsCliAliasResponse,
     Consent,
+    Credentials,
     InterpretationMetadata,
     InterpretationResponse,
     InterpretedProgram,
@@ -33,13 +34,14 @@ from ..metadata.read_only_operations_list import (
     ReadOnlyOperations,
 )
 from ..parser.lexer import split_cli_command
+from ..security.policy import PolicyDecision, SecurityPolicy
 from .driver import interpret_command as _interpret_command
 from awslabs.aws_api_mcp_server.core.common.command import IRCommand
 from awslabs.aws_api_mcp_server.core.common.helpers import operation_timer
+from fastmcp import Context
+from fastmcp.server.elicitation import AcceptedElicitation
 from io import StringIO
 from loguru import logger
-from mcp.server.elicitation import AcceptedElicitation
-from mcp.server.fastmcp import Context
 from mcp.shared.exceptions import McpError
 from mcp.types import METHOD_NOT_FOUND
 from typing import Any
@@ -50,7 +52,7 @@ async def request_consent(cli_command: str, ctx: Context):
     try:
         elicitation_result = await ctx.elicit(
             message=f"The CLI command '{cli_command}' requires explicit consent. Do you approve the execution of this command?",
-            schema=Consent,
+            response_type=Consent,
         )
 
         if (
@@ -85,6 +87,36 @@ def is_operation_read_only(ir: IRTranslation, read_only_operations: ReadOnlyOper
     return read_only_operations.has(service=service_name, operation=operation_name)
 
 
+def check_security_policy(
+    ir: IRTranslation, read_only_operations: ReadOnlyOperations, ctx: Context
+) -> PolicyDecision:
+    """Check security policy for the given command and return decision."""
+
+    def is_read_only_func(service: str, operation: str) -> bool:
+        return read_only_operations.has(service=service, operation=operation)
+
+    policy = SecurityPolicy(ctx)
+
+    # First check if this matches a customization
+    customization_decision = policy.check_customization(ir, is_read_only_func)
+    if customization_decision is not None:
+        return customization_decision
+
+    # If no customization matches, check individual operation
+    if (
+        not ir.command_metadata
+        or not getattr(ir.command_metadata, 'service_sdk_name', None)
+        or not getattr(ir.command_metadata, 'operation_sdk_name', None)
+    ):
+        return PolicyDecision.ELICIT if policy.supports_elicitation else PolicyDecision.DENY
+
+    service_name = ir.command_metadata.service_sdk_name
+    operation_name = ir.command_metadata.operation_sdk_name
+    is_read_only = is_operation_read_only(ir, read_only_operations)
+
+    return policy.determine_policy_effect(service_name, operation_name, is_read_only)
+
+
 def validate(ir: IRTranslation) -> ProgramValidationResponse:
     """Translate the given CLI command and return a validation response."""
     return ProgramValidationResponse(
@@ -94,7 +126,9 @@ def validate(ir: IRTranslation) -> ProgramValidationResponse:
 
 
 def execute_awscli_customization(
-    cli_command: str, ir_command: IRCommand
+    cli_command: str,
+    ir_command: IRCommand,
+    credentials: Credentials | None = None,
 ) -> AwsCliAliasResponse | AwsApiMcpServerErrorResponse:
     """Execute the given AWS CLI command."""
     args = split_cli_command(cli_command)[1:]
@@ -116,6 +150,7 @@ def execute_awscli_customization(
                 ir_command.operation_name,
                 ir_command.region or DEFAULT_REGION,
             ):
+                driver = get_awscli_driver(credentials)
                 driver.main(args)
 
         stdout_output = stdout_capture.getvalue()
@@ -131,14 +166,14 @@ def execute_awscli_customization(
 
 def interpret_command(
     cli_command: str,
-    default_region: str,
     max_results: int | None = None,
+    credentials: Credentials | None = None,
 ) -> ProgramInterpretationResponse:
     """Interpret the given CLI command and return an interpretation response."""
     interpreted_program = _interpret_command(
         cli_command,
-        default_region=default_region,
         max_results=max_results,
+        credentials=credentials,
     )
 
     validation_failures = (
