@@ -15,26 +15,49 @@
 import dataclasses
 from .command_metadata import CommandMetadata
 from botocore import xform_name
+from botocore.model import OperationModel
 from jmespath.parser import ParsedResult
 from typing import Any
 
 
+@dataclasses.dataclass(frozen=True)
+class OutputFile:
+    """Represents an output file configuration for AWS CLI commands."""
+
+    path: str
+    response_key: str
+
+    @classmethod
+    def from_operation(cls, path: str, operation_model: OperationModel) -> 'OutputFile':
+        """Create OutputFile from operation model for streaming operations."""
+        return cls(
+            path, response_key=getattr(operation_model.output_shape, 'serialization')['payload']
+        )
+
+
 @dataclasses.dataclass(frozen=True)
 class IRCommand:
     """Intermediate representation of an AWS CLI command."""
 
     command_metadata: CommandMetadata
     parameters: dict[str, Any]
-    region: str | None = None
+    region: str
     profile: str | None = None
     client_side_filter: ParsedResult | None = None
     is_awscli_customization: bool = False
+    output_file: OutputFile | None = None
+    endpoint_url: str | None = None
 
     @property
     def operation_python_name(self):
         """Return the Pythonic operation name for the command."""
         return xform_name(self.command_metadata.operation_sdk_name)
 
+    @property
+    def operation_cli_name(self):
+        """Return the Pythonic operation name for the command."""
+        return xform_name(self.command_metadata.operation_sdk_name).replace('_', '-')
+
     @property
     def operation_name(self):
         """Return the operation name for the command."""
