@@ -14,50 +14,63 @@
 
 import os
 import sys
+from .core.agent_scripts.manager import AGENT_SCRIPTS_MANAGER
 from .core.aws.driver import translate_cli_to_ir
 from .core.aws.service import (
+    check_security_policy,
     execute_awscli_customization,
     interpret_command,
-    is_operation_read_only,
     request_consent,
     validate,
 )
 from .core.common.config import (
     DEFAULT_REGION,
+    ENABLE_AGENT_SCRIPTS,
+    ENDPOINT_SUGGEST_AWS_COMMANDS,
     FASTMCP_LOG_LEVEL,
+    HOST,
+    PORT,
     READ_ONLY_KEY,
     READ_OPERATIONS_ONLY_MODE,
     REQUIRE_MUTATION_CONSENT,
+    STATELESS_HTTP,
+    TRANSPORT,
     WORKING_DIRECTORY,
-    get_server_directory,
 )
 from .core.common.errors import AwsApiMcpError
+from .core.common.helpers import get_requests_session, validate_aws_region
 from .core.common.models import (
     AwsApiMcpServerErrorResponse,
     AwsCliAliasResponse,
+    Credentials,
     ProgramInterpretationResponse,
 )
-from .core.kb import knowledge_base
 from .core.metadata.read_only_operations_list import ReadOnlyOperations, get_read_only_operations
-from awslabs.aws_api_mcp_server.core.common.helpers import validate_aws_region
+from .core.security.policy import PolicyDecision
 from botocore.exceptions import NoCredentialsError
+from fastmcp import Context, FastMCP
 from loguru import logger
-from mcp.server.fastmcp import Context, FastMCP
 from mcp.types import ToolAnnotations
+from pathlib import Path
 from pydantic import Field
-from typing import Annotated, Any, Optional, cast
+from typing import Annotated, Any, Optional
 
 
 logger.remove()
 logger.add(sys.stderr, level=FASTMCP_LOG_LEVEL)
 
-# Add file sink
-log_dir = get_server_directory()
-log_dir.mkdir(exist_ok=True)
+log_dir = Path.home() / '.aws' / 'aws-api-mcp'
+log_dir.mkdir(parents=True, exist_ok=True)
 log_file = log_dir / 'aws-api-mcp-server.log'
 logger.add(log_file, rotation='10 MB', retention='7 days')
 
-server = FastMCP(name='AWS-API-MCP', log_level=FASTMCP_LOG_LEVEL)
+server = FastMCP(
+    name='AWS-API-MCP',
+    log_level=FASTMCP_LOG_LEVEL,
+    host=HOST,
+    port=PORT,
+    stateless_http=STATELESS_HTTP,
+)
 READ_OPERATIONS_INDEX: Optional[ReadOnlyOperations] = None
 
 
@@ -122,7 +135,8 @@ async def suggest_aws_commands(
     query: Annotated[
         str,
         Field(
-            description="A natural language description of what you want to do in AWS. Should be detailed enough to capture the user's intent and any relevant context."
+            description="A natural language description of what you want to do in AWS. Should be detailed enough to capture the user's intent and any relevant context.",
+            max_length=2000,
         ),
     ],
     ctx: Context,
@@ -134,14 +148,22 @@ async def suggest_aws_commands(
         await ctx.error(error_message)
         return AwsApiMcpServerErrorResponse(detail=error_message)
     try:
-        suggestions = knowledge_base.get_suggestions(query)
-        logger.info(
-            'Suggested commands: {}',
-            [suggestion.get('command') for suggestion in suggestions.get('suggestions', {})],
-        )
-        return suggestions
+        with get_requests_session() as session:
+            response = session.post(
+                ENDPOINT_SUGGEST_AWS_COMMANDS,
+                json={'query': query},
+                timeout=30,
+            )
+            response.raise_for_status()
+            suggestions = response.json().get('suggestions')
+            logger.info(
+                'Suggested commands: {}',
+                [suggestion.get('command') for suggestion in suggestions],
+            )
+            return response.json()
     except Exception as e:
-        error_message = f'Error while suggesting commands: {str(e)}'
+        logger.error('Error while suggesting commands: {}', str(e))
+        error_message = 'Failed to execute tool due to internal error. Use your best judgement and existing knowledge to pick a command or point to relevant AWS Documentation.'
         await ctx.error(error_message)
         return AwsApiMcpServerErrorResponse(detail=error_message)
 
@@ -156,11 +178,12 @@ async def suggest_aws_commands(
     - All commands are validated before execution to prevent errors
     - Supports pagination control via max_results parameter
     - The current working directory is {WORKING_DIRECTORY}
+    - File paths should always have forward slash (/) as a separator regardless of the system. Example: 'c:/folder/file.txt'
 
     Best practices for command generation:
-    — Always use the most specific service and operation names
+    - Always use the most specific service and operation names
     - Always use the working directory when writing files, unless user explicitly mentioned another directory
-    — Include --region when operating across regions
+    - Include --region when operating across regions
     - Only use filters (--filters, --query, --prefix, --pattern, etc) when necessary or user explicitly asked for it
 
     Command restrictions:
@@ -197,12 +220,31 @@ async def call_aws(
     ] = None,
 ) -> ProgramInterpretationResponse | AwsApiMcpServerErrorResponse | AwsCliAliasResponse:
     """Call AWS with the given CLI command and return the result as a dictionary."""
-    logger.info('Executing AWS CLI command: {}', cli_command)
+    return await call_aws_helper(
+        cli_command=cli_command,
+        ctx=ctx,
+        max_results=max_results,
+        credentials=None,
+    )
+
+
+async def call_aws_helper(
+    cli_command: Annotated[
+        str, Field(description='The complete AWS CLI command to execute. MUST start with "aws"')
+    ],
+    ctx: Context,
+    max_results: Annotated[
+        int | None,
+        Field(description='Optional limit for number of results (useful for pagination)'),
+    ] = None,
+    credentials: Credentials | None = None,
+) -> ProgramInterpretationResponse | AwsApiMcpServerErrorResponse | AwsCliAliasResponse:
+    """Helper function that actually calls aws."""
     try:
         ir = translate_cli_to_ir(cli_command)
         ir_validation = validate(ir)
 
-        if ir_validation.validation_failed:
+        if not ir.command or ir_validation.validation_failed:
             error_message = (
                 f'Error while validating the command: {ir_validation.model_dump_json()}'
             )
@@ -223,8 +265,24 @@ async def call_aws(
             detail=error_message,
         )
 
+    logger.info(
+        'Attempting to execute AWS CLI command: aws {} {} *parameters redacted*',
+        ir.command.service_name,
+        ir.command.operation_cli_name,
+    )
+
     try:
-        if READ_OPERATIONS_INDEX is None or not is_operation_read_only(ir, READ_OPERATIONS_INDEX):
+        # Check security policy
+        if READ_OPERATIONS_INDEX is not None:
+            policy_decision = check_security_policy(ir, READ_OPERATIONS_INDEX, ctx)
+
+            if policy_decision == PolicyDecision.DENY:
+                error_message = 'Execution of this operation is denied by security policy.'
+                await ctx.error(error_message)
+                return AwsApiMcpServerErrorResponse(detail=error_message)
+            elif policy_decision == PolicyDecision.ELICIT:
+                await request_consent(cli_command, ctx)
+        else:
             if READ_OPERATIONS_ONLY_MODE:
                 error_message = (
                     'Execution of this operation is not allowed because read only mode is enabled. '
@@ -239,7 +297,11 @@ async def call_aws(
 
         if ir.command and ir.command.is_awscli_customization:
             response: AwsCliAliasResponse | AwsApiMcpServerErrorResponse = (
-                execute_awscli_customization(cli_command, ir.command)
+                execute_awscli_customization(
+                    cli_command,
+                    ir.command,
+                    credentials=credentials,
+                )
             )
             if isinstance(response, AwsApiMcpServerErrorResponse):
                 await ctx.error(response.detail)
@@ -247,8 +309,8 @@ async def call_aws(
 
         return interpret_command(
             cli_command=cli_command,
-            default_region=cast(str, DEFAULT_REGION),
             max_results=max_results,
+            credentials=credentials,
         )
     except NoCredentialsError:
         error_message = (
@@ -274,6 +336,54 @@ async def call_aws(
         )
 
 
+# EXPERIMENTAL: Agent scripts tool - only registered if ENABLE_AGENT_SCRIPTS is True
+if ENABLE_AGENT_SCRIPTS:
+
+    @server.tool(
+        name='get_execution_plan',
+        description=f"""Get the execution plan for a compiled AWS workflow. This tool provides structured, step-by-step guidance for accomplishing a complex task with AWS.
+        When a user request matches a plan intent, you MUST always call this tool to get an execution plan instead of attempting to come up with you own, since the procedures returned by this tool are more robust, and properly tested.
+
+        Below you can find the list of available scripts in the format <script_name> : <description>
+        {AGENT_SCRIPTS_MANAGER.pretty_print_scripts()}
+        INSTRUCTIONS:
+        - Call this tool with the specific script_name when user requests match the above patterns
+        - The returned plan contains detailed, tested procedures that you MUST follow exactly
+        - Do NOT attempt to create your own procedures for these tasks - use the provided plans
+        - Execute ALL steps in the plan without skipping any
+        - If plan instructions contradict each other, ask the user for guidance
+
+        Returns:
+            - Detailed script plan with step-by-step instructions for the requested task.
+        """,
+        annotations=ToolAnnotations(
+            title='Get structured execution plans for complex tasks',
+            readOnlyHint=True,
+            openWorldHint=False,
+        ),
+    )
+    async def get_execution_plan(
+        script_name: Annotated[str, Field(description='Name of the script to get the plan for')],
+        ctx: Context,
+    ) -> str | AwsApiMcpServerErrorResponse:
+        """Retrieve full script content given a script name."""
+        try:
+            script = AGENT_SCRIPTS_MANAGER.get_script(script_name)
+
+            if not script:
+                error_message = f'Script {script_name} not found'
+                logger.error(error_message)
+                raise ValueError(error_message)
+
+            logger.info(f'Retrieved script plan for {script_name}.')
+            return script.content
+
+        except Exception as e:
+            error_message = f'Error while retrieving execution plan: {str(e)}'
+            await ctx.error(error_message)
+            return AwsApiMcpServerErrorResponse(detail=error_message)
+
+
 def main():
     """Main entry point for the AWS API MCP server."""
     global READ_OPERATIONS_INDEX
@@ -295,17 +405,14 @@ def main():
     validate_aws_region(DEFAULT_REGION)
     logger.info('AWS_REGION: {}', DEFAULT_REGION)
 
+    # Always load read operations index for security policy checking
     try:
-        knowledge_base.setup()
-    except Exception as e:
-        error_message = f'Error while setting up the knowledge base: {str(e)}'
-        logger.error(error_message)
-        raise RuntimeError(error_message)
-
-    if READ_OPERATIONS_ONLY_MODE or REQUIRE_MUTATION_CONSENT:
         READ_OPERATIONS_INDEX = get_read_only_operations()
+    except Exception as e:
+        logger.warning('Failed to load read operations index: {}', e)
+        READ_OPERATIONS_INDEX = None
 
-    server.run(transport='stdio')
+    server.run(transport=TRANSPORT)
 
 
 if __name__ == '__main__':
