@@ -16,7 +16,7 @@
 SERVER="aws-api-mcp-server"
 
 # Check if the server process is running
-if pgrep -P 0 -a -l -x -f "/app/.venv/bin/python /app/.venv/bin/awslabs.$SERVER" > /dev/null; then
+if pgrep -P 0 -a -l -x -f "/app/.venv/bin/python3? /app/.venv/bin/awslabs.$SERVER" > /dev/null; then
   echo -n "$SERVER is running";
   exit 0;
 fi;
