@@ -1,10 +1,11 @@
 import pytest
 import re
 from awslabs.aws_api_mcp_server.core.common.command_metadata import CommandMetadata
+from awslabs.aws_api_mcp_server.core.common.config import WORKING_DIRECTORY
 from awslabs.aws_api_mcp_server.core.common.errors import (
     ClientSideFilterError,
-    CommandValidationError,
     ExpectedArgumentError,
+    FileParameterError,
     InvalidChoiceForParameterError,
     InvalidParametersReceivedError,
     InvalidServiceError,
@@ -19,7 +20,22 @@ from awslabs.aws_api_mcp_server.core.common.errors import (
     ShortHandParserError,
     UnknownFiltersError,
 )
-from awslabs.aws_api_mcp_server.core.parser.parser import parse
+from awslabs.aws_api_mcp_server.core.parser.parser import (
+    _validate_endpoint,
+    _validate_output_file,
+    parse,
+)
+from unittest.mock import Mock, patch
+
+
+@pytest.mark.parametrize(
+    'command',
+    [('aws organizations describe-organization')],
+)
+def test_service_not_expecting_parameters(command):
+    """Test that parsing of commands that do not expect any parameters succeeds."""
+    ir = parse(command)
+    assert ir.parameters == {}
 
 
 @pytest.mark.parametrize(
@@ -216,25 +232,6 @@ def test_invalid_type_for_parameter(command, message):
         parse(command)
 
 
-@pytest.mark.parametrize(
-    'command, message',
-    [
-        (
-            'aws lambda  update-function-code --function-name MyFunction --zip-file  fileb://newfunction.zip',
-            str(
-                CommandValidationError(
-                    '-zip-file must be a zip file with the fileb:// prefix.\nExample usage:  --zip-file fileb://path/to/file.zip'
-                )
-            ),
-        )
-    ],
-)
-def test_command_validation_error_for_parameter(command, message):
-    """Test that a command validation error is raised for invalid parameters."""
-    with pytest.raises(CommandValidationError, match=message):
-        parse(command)
-
-
 @pytest.mark.parametrize(
     'command, messages',
     [
@@ -411,7 +408,6 @@ def test_plural_singular_params(command):
         "aws ec2 describe-availability-zones --query='AvailabilityZones[?ZoneName==`us-east-1a`]'",
         'aws s3api get-bucket-lifecycle --bucket my-s3-bucket',
         'aws --region=us-east-1 ec2 get-subnet-cidr-reservations --subnet-id subnet-012 --color=on',
-        'aws s3api get-object --bucket aws-sam-cli-managed-default-samclisourcebucket --key lambda-sqs-sam-test-1/1f1a15295b5529effed491b54a5b5b83.template -',
         "aws apigateway get-export --parameters extensions='postman' --rest-api-id a1b2c3d4e5 --stage-name dev --export-type swagger -",
     ],
 )
@@ -420,6 +416,16 @@ def test_should_pass_for_valid_equal_sign_params(command):
     parse(command)
 
 
+@patch(
+    'awslabs.aws_api_mcp_server.core.common.file_system_controls.ALLOW_UNRESTRICTED_LOCAL_FILE_ACCESS',
+    True,
+)
+def test_should_pass_for_valid_equal_sign_params_with_file_output():
+    """Test that valid equal sign parameters with file output are accepted when unrestricted access is enabled."""
+    command = f'aws s3api get-object --bucket aws-sam-cli-managed-default-samclisourcebucket --key lambda-sqs-sam-test-1/1f1a15295b5529effed491b54a5b5b83.template {WORKING_DIRECTORY}/output.template'
+    parse(command)
+
+
 @pytest.mark.parametrize(
     'command',
     # All these are valid
@@ -605,35 +611,205 @@ def test_client_side_filter_error():
         parse(command)
 
 
-@pytest.mark.parametrize(
-    'command',
-    [
-        'aws s3api get-object --bucket aws-sam-cli-managed-default-samclisourcebucket --key lambda-sqs-sam-test-1/1f1a15295b5529effed491b54a5b5b83.template myfile.template',
-        'aws lambda invoke --function-name my-function response.json',
-    ],
-)
-def test_outfile_parameter_not_supported(command):
-    """Test that outfile parameters raise a validation error."""
-    with pytest.raises(
-        CommandValidationError,
-        match='Output file parameters are not supported yet. Use - as the output file to get the requested data in the response.',
-    ):
-        parse(command)
-
-
 def test_valid_expand_user_home_directory():
-    """Test that tilde is replaced with user home directory."""
-    result = parse(cli_command='aws s3 cp s3://my_file ~/temp/test.txt')
-    assert not any(param.startswith('~') for param in result.parameters['--paths'])
+    """Test that tilde or user home directory is invalid path."""
+    with pytest.raises(ValueError) as exc_info:
+        parse(cli_command='aws s3 cp s3://my_file ~/temp/test.txt')
+
+    error_message = str(exc_info.value)
+    assert 'is outside the allowed working directory' in error_message
+    assert (
+        'Set AWS_API_MCP_ALLOW_UNRESTRICTED_LOCAL_FILE_ACCESS=true to allow unrestricted file access'
+        in error_message
+    )
 
 
 def test_invalid_expand_user_home_directory():
-    """Test that tilde is not replaced."""
-    result = parse(cli_command='aws s3 cp s3://my_file ~user_that_does_not_exist/temp/test.txt')
-    assert any(param.startswith('~') for param in result.parameters['--paths'])
+    """Test that tilde is not expanded."""
+    expected_message = (
+        "Invalid file parameter '~user_that_does_not_exist/temp/test.txt' for service 's3' and operation 'cp': "
+        'should be an absolute path. Please provide a valid file path.'
+    )
+    with pytest.raises(FileParameterError) as exc_info:
+        parse(cli_command='aws s3 cp s3://my_file ~user_that_does_not_exist/temp/test.txt')
+
+    assert str(exc_info.value) == expected_message
 
 
-def test_profile():
+@patch('boto3.Session')
+def test_profile(mock_boto3_session):
     """Test that the profile is correctly extracted."""
+    mock_session_instance = mock_boto3_session.return_value
+    mock_session_instance.region_name = 'us-east-1'
+
     result = parse(cli_command='aws s3api list-buckets --profile test-profile')
     assert result.profile == 'test-profile'
+    mock_boto3_session.assert_called_with(profile_name='test-profile')
+
+
+@pytest.mark.parametrize(
+    'command,expected_service,expected_operation,expected_file_path',
+    [
+        (
+            'aws s3api get-object --bucket test-bucket --key test-key relative/path/file.txt',
+            's3',
+            'GetObject',
+            'relative/path/file.txt',
+        ),
+        (
+            'aws lambda invoke --function-name my-function response.json',
+            'lambda',
+            'Invoke',
+            'response.json',
+        ),
+    ],
+)
+def test_validate_output_file_raises_error_for_relative_paths(
+    command, expected_service, expected_operation, expected_file_path
+):
+    """Test that _validate_output_file raises FileParameterError for streaming operations with relative paths."""
+    expected_message = f'{expected_file_path} should be an absolute path'
+    with pytest.raises(ValueError, match=expected_message):
+        parse(command)
+
+
+def test_validate_output_file_no_streaming_output():
+    """Test that _validate_output_file does nothing when has_streaming_output is False."""
+    command_metadata = Mock()
+    command_metadata.has_streaming_output = False
+    parsed_args = Mock()
+
+    # Should not raise any exception
+    _validate_output_file(command_metadata, parsed_args)
+
+
+def test_validate_output_file_with_dash_output():
+    """Test that _validate_output_file allows '-' as output file."""
+    command_metadata = Mock()
+    command_metadata.has_streaming_output = True
+
+    operation_args = Mock()
+    operation_args.outfile = '-'
+
+    parsed_args = Mock()
+    parsed_args.operation_args = operation_args
+
+    # Should not raise any exception
+    _validate_output_file(command_metadata, parsed_args)
+
+
+def test_validate_output_file_with_relative_path():
+    """Test that _validate_output_file raises ValueError for relative paths."""
+    command_metadata = Mock()
+    command_metadata.has_streaming_output = True
+
+    operation_args = Mock()
+    operation_args.outfile = 'relative/path.txt'
+
+    parsed_args = Mock()
+    parsed_args.operation_args = operation_args
+
+    with pytest.raises(ValueError, match='relative/path.txt should be an absolute path'):
+        _validate_output_file(command_metadata, parsed_args)
+
+
+@patch('awslabs.aws_api_mcp_server.core.parser.parser.validate_file_path')
+def test_validate_output_file_with_absolute_path(mock_validate_file_path):
+    """Test that _validate_output_file calls validate_file_path for absolute paths."""
+    command_metadata = Mock()
+    command_metadata.has_streaming_output = True
+
+    operation_args = Mock()
+    operation_args.outfile = '/absolute/path.txt'
+
+    parsed_args = Mock()
+    parsed_args.operation_args = operation_args
+
+    _validate_output_file(command_metadata, parsed_args)
+
+    mock_validate_file_path.assert_called_once_with('/absolute/path.txt')
+
+
+@patch('awslabs.aws_api_mcp_server.core.parser.parser.validate_file_path')
+def test_validate_output_file_propagates_validate_file_path_error(mock_validate_file_path):
+    """Test that _validate_output_file propagates errors from validate_file_path."""
+    command_metadata = Mock()
+    command_metadata.has_streaming_output = True
+
+    operation_args = Mock()
+    operation_args.outfile = '/absolute/path.txt'
+
+    parsed_args = Mock()
+    parsed_args.operation_args = operation_args
+
+    mock_validate_file_path.side_effect = ValueError('File validation error')
+
+    with pytest.raises(ValueError, match='File validation error'):
+        _validate_output_file(command_metadata, parsed_args)
+
+
+@pytest.mark.parametrize(
+    'endpoint',
+    [
+        None,
+        '',
+        'localhost:8080',
+        'http://localhost:8080',
+        'https://localhost:8080',
+        '127.0.0.1:8080',
+        'http://127.0.0.1:8080',
+        'http://[::1]:8080',
+    ],
+)
+def test_validate_endpoint_valid_loopback(endpoint):
+    """Test that valid loopback endpoints are accepted."""
+    _validate_endpoint(endpoint)
+
+
+@pytest.mark.parametrize(
+    'endpoint,expected_error',
+    [
+        ('localhost:invalid_port', 'Invalid endpoint or port'),
+        ('http://localhost:abc', 'Invalid endpoint or port'),
+        ('://invalid', 'Could not find hostname'),
+        ('http://', 'Could not find hostname'),
+        ('192.168.1.1:8080', 'Local endpoint was not a loopback address'),
+        ('http://192.168.1.1:8080', 'Local endpoint was not a loopback address'),
+        ('google.com:8080', 'Could not resolve endpoint'),
+        ('https://google.com', 'Could not resolve endpoint'),
+        ('example.com', 'Could not resolve endpoint'),
+        ('::1:8080', 'Invalid endpoint or port'),  # IPv6 without brackets
+    ],
+)
+def test_validate_endpoint_invalid(endpoint, expected_error):
+    """Test that invalid endpoints raise appropriate ValueError."""
+    with pytest.raises(ValueError, match=expected_error):
+        _validate_endpoint(endpoint)
+
+
+def test_validate_endpoint_empty_string():
+    """Test that empty string is handled like None."""
+    _validate_endpoint('')
+
+
+def test_validate_endpoint_localhost_conversion():
+    """Test that localhost is converted to 127.0.0.1."""
+    _validate_endpoint('localhost:8080')
+
+
+def test_validate_endpoint_ipv6_loopback():
+    """Test that IPv6 loopback addresses are accepted."""
+    _validate_endpoint('[::1]:8080')
+    _validate_endpoint('http://[::1]:8080')
+
+
+def test_validate_endpoint_protocol_handling():
+    """Test that endpoints with and without protocol are handled correctly."""
+    _validate_endpoint('localhost:8080')
+    _validate_endpoint('https://localhost:8080')
+
+
+def test_validate_endpoint_non_http_protocols():
+    """Test that non-HTTP protocols with localhost are accepted."""
+    _validate_endpoint('ftp://localhost:8080')
+    _validate_endpoint('ws://127.0.0.1:8080')
